@@ -19,13 +19,15 @@
 #include <gwenhywfar/ctplugin_be.h>
 #include <gwenhywfar/text.h>
 #include <gwenhywfar/padd.h>
+#include <gwenhywfar/gui.h>
+#include <gwenhywfar/i18n.h>
 
 #include <chipcard/client/cards/starcos.h>
 #include <chipcard/client/cards/processorcard.h>
 #include <chipcard/client/ct/ct_card.h>
 
 
-#define I18N(msg) msg
+#define I18N(message) GWEN_I18N_Translate("libchipcard", message)
 
 
 
@@ -119,6 +121,7 @@ LC_Crypt_TokenStarcos_Plugin_CheckToken(GWEN_PLUGIN *pl,
   LC_CLIENT_RESULT res;
   LC_CARD *hcard=0;
   const char *currCardNumber;
+  int i;
 
   assert(pl);
   cpl=GWEN_INHERIT_GETDATA(GWEN_PLUGIN, LC_CT_PLUGIN_STARCOS, pl);
@@ -133,7 +136,43 @@ LC_Crypt_TokenStarcos_Plugin_CheckToken(GWEN_PLUGIN *pl,
     return GWEN_ERROR_IO;
   }
 
-  res=LC_Client_GetNextCard(cpl->client, &hcard, 5);
+  for (i=0;i<10;i++) {
+    res=LC_Client_GetNextCard(cpl->client, &hcard, i==0?5:10);
+    if (res==LC_Client_ResultOk)
+      break;
+    else {
+      if (res==LC_Client_ResultWait) {
+	int mres;
+
+	mres=GWEN_Gui_MessageBox(GWEN_GUI_MSG_FLAGS_SEVERITY_NORMAL |
+				 GWEN_GUI_MSG_FLAGS_CONFIRM_B1 |
+				 GWEN_GUI_MSG_FLAGS_TYPE_INFO,
+				 I18N("Insert card"),
+				 I18N("Please insert a chipcard into the reader "
+				      "and click a button."
+				      "<html>"
+				      "Please insert a chipcard into the reader "
+				      "and click a button."
+				      "</html>"),
+				 I18N("Ok"),
+				 I18N("Abort"),
+				 NULL,
+				 0);
+	if (mres!=1) {
+	  DBG_ERROR(LC_LOGDOMAIN, "Error in user interaction (%d)", mres);
+	  LC_Client_Stop(cpl->client);
+	  return GWEN_ERROR_USER_ABORTED;
+	}
+      }
+      else {
+	GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
+			     I18N("Error while waiting for card"));
+	LC_Client_Stop(cpl->client);
+	return GWEN_ERROR_IO;
+      }
+    }
+  }
+
   if (res!=LC_Client_ResultOk) {
     DBG_ERROR(LC_LOGDOMAIN,
 	      "No card within specified timeout (%d)", res);
@@ -894,8 +933,16 @@ LC_Crypt_TokenStarcos_SetKeyInfo(GWEN_CRYPT_TOKEN *ct,
 	GWEN_Crypt_Token_KeyInfo_SetStatus(cardki, i);
       }
     }
+#else
+    LC_Starcos_KeyDescr_SetStatus(descr, LC_STARCOS_KEY_STATUS_ACTIVE);
 #endif
 
+    if ((kid>=0x86 && kid<=0x8a) ||
+	(kid>=0x96 && kid<=0x9a))
+      LC_Starcos_KeyDescr_SetKeyType(descr, 'V');
+    else
+      LC_Starcos_KeyDescr_SetKeyType(descr, 'S');
+
     /* save descriptor after changing */
     DBG_INFO(LC_LOGDOMAIN, "Saving key descriptor");
     res=LC_Starcos_SaveKeyDescr(lct->card, descr);
@@ -1043,18 +1090,18 @@ LC_Crypt_TokenStarcos_GetContext(GWEN_CRYPT_TOKEN *ct,
       default:
 	break;
       }
-
-      /* set key ids (same for every context) */
-      GWEN_Crypt_Token_Context_SetSignKeyId(ctx, 0x80+id);     /* user sign key */
-      GWEN_Crypt_Token_Context_SetVerifyKeyId(ctx, 0x90+id);   /* peer sign key */
-      GWEN_Crypt_Token_Context_SetEncipherKeyId(ctx, 0x95+id); /* peer crypt key */
-      GWEN_Crypt_Token_Context_SetDecipherKeyId(ctx, 0x85+id); /* user crypt key */
     }
     else {
       DBG_INFO(LC_LOGDOMAIN, "Empty entry (%d)", id);
     }
     GWEN_DB_Group_free(dbData);
 
+    /* set key ids (same for every context) */
+    GWEN_Crypt_Token_Context_SetSignKeyId(ctx, 0x80+id);     /* user sign key */
+    GWEN_Crypt_Token_Context_SetVerifyKeyId(ctx, 0x90+id);   /* peer sign key */
+    GWEN_Crypt_Token_Context_SetEncipherKeyId(ctx, 0x95+id); /* peer crypt key */
+    GWEN_Crypt_Token_Context_SetDecipherKeyId(ctx, 0x85+id); /* user crypt key */
+
     /* store new context */
     lct->contexts[id-1]=ctx;
   }
@@ -1132,11 +1179,19 @@ LC_Crypt_TokenStarcos_GenerateKey(GWEN_CRYPT_TOKEN *ct,
     return GWEN_ERROR_IO;
   }
 
+  LC_Starcos_KeyDescr_SetStatus(kdescr, LC_STARCOS_KEY_STATUS_INACTIVE_FREE);
+  res=LC_Starcos_SaveKeyDescr(lct->card, kdescr);
+  if (res!=LC_Client_ResultOk) {
+    DBG_INFO(LC_LOGDOMAIN, "here (%d)", res);
+    LC_Starcos_KeyDescr_free(kdescr);
+    return GWEN_ERROR_IO;
+  }
+
   if (nkeyId==0x8f)
     LC_Starcos_KeyDescr_SetKeyType(kdescr, 'S');
   else
     LC_Starcos_KeyDescr_SetKeyType(kdescr, 'V');
-  LC_Starcos_KeyDescr_SetStatus(kdescr, 0x10);
+  LC_Starcos_KeyDescr_SetStatus(kdescr, LC_STARCOS_KEY_STATUS_ACTIVE);
 
   res=LC_Starcos_ActivateKeyPair(lct->card, nkeyId, kid, kdescr);
   if (res!=LC_Client_ResultOk) {
@@ -1204,7 +1259,7 @@ LC_Crypt_TokenStarcos_Sign(GWEN_CRYPT_TOKEN *ct,
   res=LC_Card_IsoManageSe(lct->card, 0xb6,
                           kid & 0xff,
                           kid & 0xff,
-			  0x21); /* assume RMD160 and 9796-2 */
+			  0x25); /* assume RMD160 and 9796-2 */
   if (res!=LC_Client_ResultOk) {
     DBG_ERROR(LC_LOGDOMAIN, "Error preparing signing (%d)", res);
     return GWEN_ERROR_IO;
@@ -1223,7 +1278,7 @@ LC_Crypt_TokenStarcos_Sign(GWEN_CRYPT_TOKEN *ct,
   }
 
   /* copy signature into given buffer */
-  if (GWEN_Buffer_GetUsedBytes(dbuf)>=*pSignatureLen) {
+  if (GWEN_Buffer_GetUsedBytes(dbuf)>*pSignatureLen) {
     DBG_ERROR(LC_LOGDOMAIN, "Buffer overrun (%d>=%d)",
 	      GWEN_Buffer_GetUsedBytes(dbuf), *pSignatureLen);
     GWEN_Buffer_free(dbuf);
@@ -1289,11 +1344,11 @@ LC_Crypt_TokenStarcos_Verify(GWEN_CRYPT_TOKEN *ct,
 
   /* set security environment */
   res=LC_Card_IsoManageSe(lct->card, 0xb6,
+			  0,
                           kid & 0xff,
-                          kid & 0xff,
-			  0x21); /* assume RMD160 and 9796-2 */
+			  0x25); /* assume RMD160 and 9796-2 */
   if (res!=LC_Client_ResultOk) {
-    DBG_ERROR(LC_LOGDOMAIN, "Error preparing signing (%d)", res);
+    DBG_ERROR(LC_LOGDOMAIN, "Error preparing verification (%d)", res);
     return GWEN_ERROR_IO;
   }
 
@@ -1427,8 +1482,10 @@ LC_Crypt_TokenStarcos_Decipher(GWEN_CRYPT_TOKEN *ct,
   lct=GWEN_INHERIT_GETDATA(GWEN_CRYPT_TOKEN, LC_CT_STARCOS, ct);
   assert(lct);
 
-  if (GWEN_Crypt_PaddAlgo_GetId(a)!=GWEN_Crypt_PaddAlgoId_LeftZero) {
-    DBG_ERROR(LC_LOGDOMAIN, "Invalid padd algo");
+  if (GWEN_Crypt_PaddAlgo_GetId(a)!=GWEN_Crypt_PaddAlgoId_LeftZero &&
+      GWEN_Crypt_PaddAlgo_GetId(a)!=GWEN_Crypt_PaddAlgoId_None) {
+    DBG_ERROR(LC_LOGDOMAIN, "Invalid padd algo (%02x)",
+	      GWEN_Crypt_PaddAlgo_GetId(a));
     return GWEN_ERROR_INVALID;
   }
 
@@ -1471,6 +1528,13 @@ LC_Crypt_TokenStarcos_Decipher(GWEN_CRYPT_TOKEN *ct,
     return GWEN_ERROR_IO;
   }
 
+  if (GWEN_Crypt_PaddAlgo_GetId(a)==GWEN_Crypt_PaddAlgoId_None &&
+      GWEN_Buffer_GetUsedBytes(dbuf)<inLen) {
+    GWEN_Buffer_SetPos(dbuf, 0);
+    GWEN_Buffer_FillLeftWithBytes(dbuf, 0,
+				  inLen-GWEN_Buffer_GetUsedBytes(dbuf));
+  }
+
   /* copy result into given buffer */
   if (GWEN_Buffer_GetUsedBytes(dbuf)>=*pOutLen) {
     DBG_ERROR(LC_LOGDOMAIN, "Buffer overrun (%d>=%d)",
