@@ -285,6 +285,9 @@ const char *DriverIFD_GetErrorText(LCD_DRIVER *d, uint32_t err) {
   case CCID_ICC_NOT_PRESENT:
     s="Card not present";
     break;
+  case CCID_COMMUNICATION_ERROR:
+    s="Communication error";
+    break;
   default:
     s="Unknow error code";
   };
@@ -511,7 +514,7 @@ uint32_t DriverIFD_SendAPDU(LCD_DRIVER *d,
 
 
 
-long DriverIFD__ConnectSlot(LCD_DRIVER *d, LCD_SLOT *sl) {
+uint32_t DriverIFD_ConnectSlot(LCD_DRIVER *d, LCD_SLOT *sl) {
   long retval;
   unsigned char atrBuffer[300];
   DWORD atrLen;
@@ -567,6 +570,7 @@ long DriverIFD__ConnectSlot(LCD_DRIVER *d, LCD_SLOT *sl) {
 
   if (retval==CCID_ICC_PRESENT || retval==0) {
     int proto;
+    int ccid_proto=0;
 
     DBG_INFO(LCD_Reader_GetLogger(r), "Card present.");
     LCD_Slot_AddStatus(sl, LCD_SLOT_STATUS_CARD_INSERTED);
@@ -584,6 +588,26 @@ long DriverIFD__ConnectSlot(LCD_DRIVER *d, LCD_SLOT *sl) {
       LCD_Slot_AddFlags(sl, LCD_SLOT_FLAGS_PROCESSORCARD);
     }
     LCD_Slot_SetProtocolInfo(sl, proto);
+
+    switch (proto) {
+      case 0: ccid_proto = 1; break;
+      case 1: ccid_proto = 2; break;
+      default:
+	DBG_ERROR(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
+	         "CCID: Unkown Protocol in ATR: T=%d", proto);
+	break;
+    }
+    DBG_INFO(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
+	     "CCID: Setting Protocol T=%d", proto);
+    retval=dct->setProtoFn(LCD_Slot_GetSlotNum(sl),
+			   ccid_proto, 0, 0, 0, 0);
+    if (retval) {
+      DBG_ERROR(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
+	       "CCID: Error setting protocol T=%d (%d)",
+	       proto, (int)retval);
+      return retval;
+    }
+
   }
   else if (retval==CCID_ICC_NOT_PRESENT) {
     DBG_NOTICE(LCD_Reader_GetLogger(r), "No card inserted");
@@ -602,55 +626,6 @@ long DriverIFD__ConnectSlot(LCD_DRIVER *d, LCD_SLOT *sl) {
 
 
 
-uint32_t DriverIFD_ConnectSlot(LCD_DRIVER *d, LCD_SLOT *sl) {
-  int i;
-  DRIVER_IFD *dct;
-  long retval;
-
-  assert(d);
-  dct=GWEN_INHERIT_GETDATA(LCD_DRIVER, DRIVER_IFD, d);
-  assert(dct);
-
-  retval=DriverIFD__ConnectSlot(d, sl);
-  if (retval!=CCID_ERROR_POWER_ACTION) {
-    DBG_INFO(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
-	     "CCID: Result is not POWER_ACTION (%d)",
-	     (int)retval);
-    return retval;
-  }
-
-  if (dct->setProtoFn==NULL)
-    return retval;
-
-  /* try protocols 0 through 15 */
-  for (i=0; i<16; i++) {
-    /* set protocol bt don't change PTS */
-    retval=dct->setProtoFn(LCD_Slot_GetSlotNum(sl),
-			   i, 0, 0, 0, 0);
-    if (retval) {
-      DBG_INFO(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
-	       "CCID: Error setting protocol %d (%d)",
-	       i, (int)retval);
-    }
-    else {
-      retval=DriverIFD__ConnectSlot(d, sl);
-      if (retval!=CCID_ERROR_POWER_ACTION) {
-	DBG_INFO(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
-		 "CCID: Result is not POWER_ACTION (%d, proto=%d)",
-		 (int)retval, i);
-	return retval;
-      }
-    }
-  }
-
-  /* assuming no card inserted */
-  DBG_INFO(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
-	   "Assuming no card is inserted");
-  return 0;
-}
-
-
-
 uint32_t DriverIFD_DisconnectSlot(LCD_DRIVER *d, LCD_SLOT *sl) {
   long retval;
   unsigned char atrBuffer[300];
@@ -714,8 +689,32 @@ uint32_t DriverIFD_ResetSlot(LCD_DRIVER *d, LCD_SLOT *sl) {
     LCD_Slot_SubStatus(sl, LCD_SLOT_STATUS_CARD_CONNECTED);
   }
   else if (retval==0) {
+    int proto;
+    int ccid_proto=0;
+
     LCD_Slot_AddStatus(sl, LCD_SLOT_STATUS_CARD_INSERTED);
     LCD_Slot_AddStatus(sl, LCD_SLOT_STATUS_CARD_CONNECTED);
+
+    proto=DriverIFD_ExtractProtocolInfo(atrBuffer, atrLen);
+    // Or should it be read from LCD_Slot_GetProtocolInfo?
+    switch (proto) {
+      case 0: ccid_proto = 1; break;
+      case 1: ccid_proto = 2; break;
+      default:
+	DBG_ERROR(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
+	         "CCID: Unkown Protocol in ATR: T=%d", proto);
+	break;
+    }
+    DBG_INFO(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
+	     "CCID: Setting Protocol T=%d", proto);
+    retval=dct->setProtoFn(LCD_Slot_GetSlotNum(sl),
+			   ccid_proto, 0, 0, 0, 0);
+    if (retval) {
+      DBG_ERROR(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
+	       "CCID: Error setting protocol T=%d (%d)",
+	       proto, (int)retval);
+      return retval;
+    }
   }
   else {
     DBG_NOTICE(LCD_Reader_GetLogger(LCD_Slot_GetReader(sl)),
@@ -864,10 +863,10 @@ uint32_t DriverIFD_ReaderInfo(LCD_DRIVER *d, LCD_READER *r, GWEN_BUFFER *buf){
 #ifdef LC_ENDIAN_LITTLE
       /* only translate control codes for generic ccid driver >=1.1.0 */
       if (lc_driver__ccid_pre_1_1_0==-1)
-        v=((v & 0xff000000)>>24) |
-          ((v & 0x00ff0000)>>8) |
-          ((v & 0x0000ff00)<<8) |
-          ((v & 0x000000ff)<<24);
+	v=((v & 0xff000000)>>24) |
+	  ((v & 0x00ff0000)>>8) |
+	  ((v & 0x0000ff00)<<8) |
+	  ((v & 0x000000ff)<<24);
 #endif
       DBG_NOTICE(lg, "TLV: %d (%08x)", tlv[i].tag, v);
       ReaderIFD_SetFeatureCode(r, tlv[i].tag, v);
@@ -1018,15 +1017,15 @@ void log_msg(const int priority, const char *fmt, ...) {
 
   switch(priority) {
   case 1: /* PCSC_LOG_INFO */
-    DBG_INFO(0, "PCSC: %s", msgBuf);
+    DBG_INFO(0, "PCSC(%d): %s", priority, msgBuf);
     break;
   case 2: /* PCSC_LOG_ERROR */
   case 3: /* PCSC_LOG_CRITICAL */
-    DBG_ERROR(0, "PCSC: %s", msgBuf);
+    DBG_ERROR(0, "PCSC(%d): %s", priority, msgBuf);
     break;
   case 0: /* PCSC_LOG_DEBUG */
   default:
-    DBG_DEBUG(0, "PCSC: %s", msgBuf);
+    DBG_DEBUG(0, "PCSC(%d): %s", priority, msgBuf);
     break;
   } /* switch */
 
@@ -1104,14 +1103,14 @@ char *pcsc_stringify_error(long x) {
 void DriverIFD__checkMsg(int priority, const char *msg) {
   if (lc_driver__ccid_pre_1_1_0==0 && priority>0) {
     if (GWEN_Text_ComparePattern((const char*)msg,
-                                 "*ProductString: Generic CCID driver v*",
+                                 "*ProductString: Generic CCID driver*",
                                  0)!=-1) {
       const char *p;
 
       p=msg;
       p=strrchr(p, 'v');
-      if (p) {
-        int vmajor=0;
+      if (p && isdigit(p[1])) {
+	int vmajor=0;
         int vminor=0;
         int vpatchlevel=0;
 
@@ -1152,6 +1151,13 @@ void DriverIFD__checkMsg(int priority, const char *msg) {
           }
         }
       }
+      else {
+	/* grr... newer drivers don't show the version string in this message,
+	 * so we assume this *is* a newer driver
+	 */
+	DBG_NOTICE(0, "Message doesn't show the version, assuming newer driver version (post 1.1.0)");
+	lc_driver__ccid_pre_1_1_0=-1;
+      }
     } /* if generic ccid driver */
   } /* if driver type still undetermined */
 }
