@@ -15,6 +15,9 @@
 #define CHIPCARD_CARD_KVKCARD_H
 
 #include <chipcard/client/card.h>
+#include <chipcard/client/cards/hipersonaldata.h>
+#include <chipcard/client/cards/hiinsurancedata.h>
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -77,8 +80,17 @@ LC_CLIENT_RESULT LC_KVKCard_Reopen(LC_CARD *card);
  * The card object remains the owner od the object returned (if any), so you
  * must not manipulate or destroy it.
  */
-CHIPCARD_API
+CHIPCARD_API CHIPCARD_DEPRECATED
 GWEN_DB_NODE *LC_KVKCard_GetCardData(const LC_CARD *card);
+
+CHIPCARD_API
+LC_CLIENT_RESULT LC_KvkCard_ReadCardData(LC_CARD *card,
+					 LC_HI_PERSONAL_DATA **pPersonal,
+					 LC_HI_INSURANCE_DATA **pInsurance);
+
+CHIPCARD_API
+const char *LC_KvkCard_GetCardNumber(const LC_CARD *card);
+
 /*@}*/
 
 /*@}*/ /* defgroup */
