@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: examplecard.h 378 2007-05-02 08:41:05Z martin $
+    cvs         : $Id$
     begin       : Sun Jun 13 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -33,6 +33,8 @@ LC_CLIENT_RESULT ExampleCard_Reopen(LC_CARD *card);
 int ExampleCard_GetExampleData(const LC_CARD *card);
 
 
+typedef struct TYPE_REAL TYPE_VISIBLE;
+int test_type(TYPE_VISIBLE *v);
 
 #endif /* CHIPCARD_CARD_EXAMPLECARD_P_H */
 
